/*
 - parallelplugin.c
 - by Boris LESNER <boris@lesner.org>
 - basics parallel port manipulation routines
 - please note that this file is not specifically intended
   for use with squeak, it's been desingned for general purpose

This software is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; version 2 dated June, 1991.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this software; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <string.h> /* for memset */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sys/ioctl.h>
#include <linux/ppdev.h> // contient PPRDATA, ...
/*
 * Read/write data
 * #define PPRDATA		_IOR(PP_IOCTL, 0x85, unsigned char)
 * #define PPWDATA		_IOW(PP_IOCTL, 0x86, unsigned char)
 * 
 */

#include "parallelplugin.h"

#define MODULE_NAME "parallelplugin 14 April 2005 (e)"

int fd;

char* getModuleName()
{
	return MODULE_NAME;
}

static unsigned int two_power(unsigned int n)
{
	unsigned int i, p = 1;
	
	for(i = 0; i < n; i++)
		p *= 2;
	
	return p;
}

char *bin_to_str(unsigned int num, int n)
{
	char *bin_num;
	int bit_idx;
 
	bin_num = (char*)malloc((n+1)*sizeof(char));

	memset(bin_num, 0, n);
	bin_num[n] = '\0';
	
	for(bit_idx = 0; bit_idx < n; bit_idx++)
		bin_num[n-1-bit_idx] = ((num & two_power(bit_idx))>>bit_idx) + '0';
	
	return bin_num;
}

int open_port(char *device)
{
	if((fd = open(device, O_RDWR)) == -1) 
	{
		perror(device);
		close(fd);
		return -1;
	}

	if(ioctl(fd, PPCLAIM))
	{
		perror("ioctl: PPCLAIM");
		close(fd);
		return -1;
	}

	return fd;
}

void close_port(void)
{
	close(fd);
}

int read_data(char *data)
{
	if((ioctl(fd, PPRDATA, data)) != 0)
	{
		perror("ioctl: PPRDATA");
		data = NULL;
		return -1;
	}

	return 0;
}


int write_data(char data)
{
	if(ioctl(fd, PPWDATA, &data) == -1)
	{
		perror("ioctl: PPWDATA");
		return -1;
	}
	return 0;
} 

int read_control(char *ctl)
{
	if(ioctl(fd, PPRCONTROL, ctl) == -1)
	{
		perror("ioctl: PPRCONTROL");
		ctl = NULL;
		return -1;
	}

	return 0;
}

int write_control(char ctl)
{
	if(ioctl(fd, PPWCONTROL, &ctl) == -1)
	{
		perror("ioctl: PPWCONTROL");
		return -1;
	}

	return 0;
} 

int read_status(char *status)
{
	if(ioctl(fd, PPRSTATUS, status) == -1)
	{
		perror("ioctl: PPRSTATUS");
		status = NULL;
		return -1;
	}

	return 0;
}
